"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@lincode/utils");
const react_1 = require("react");
exports.default = (cb, disabled) => {
    const [el, setEl] = (0, react_1.useState)(null);
    const [dragOver, setDragOver] = (0, react_1.useState)(false);
    (0, react_1.useEffect)(() => {
        if (!el || disabled)
            return;
        const setDragOverDebounced = (0, utils_1.debounce)((value) => setDragOver(value), 100, "trailing");
        let valueOld = false;
        const setDragOverTrack = (value) => {
            if (valueOld === value)
                return;
            valueOld = value;
            setDragOverDebounced(value);
        };
        const dragOverCb = (e) => {
            e.preventDefault();
            setDragOverTrack(true);
        };
        el.addEventListener("dragover", dragOverCb);
        const dragEnterCb = (e) => {
            e.preventDefault();
            setDragOverTrack(true);
        };
        el.addEventListener("dragenter", dragEnterCb);
        const dragLeaveCb = (e) => {
            e.preventDefault();
            setDragOverTrack(false);
        };
        el.addEventListener("dragleave", dragLeaveCb);
        const dropCb = (e) => {
            e.preventDefault();
            setDragOverTrack(false);
            cb(e);
        };
        el.addEventListener("drop", dropCb);
        return () => {
            el.removeEventListener("dragover", dragOverCb);
            el.removeEventListener("dragenter", dragEnterCb);
            el.removeEventListener("dragleave", dragLeaveCb);
            el.removeEventListener("drop", dropCb);
        };
    }, [el, disabled]);
    return [setEl, dragOver];
};
