import { useLayoutEffect, useState } from "react"
import ResizeObserver from "resize-observer-polyfill"

const zeroBounds = new DOMRect()

export default (cb?: (size: readonly [number, number, DOMRect]) => void) => {
    const [el, setEl] = useState<HTMLElement | null>(null)
    const [size, setSize] = useState<readonly [number, number, DOMRect]>([0, 0, zeroBounds])

    useLayoutEffect(() => {
        if (!el) return
        
        const resizeObserver = new ResizeObserver(() => {
            const size = <const>[el.clientWidth, el.clientHeight, el.getBoundingClientRect()]
            cb?.(size)
            setSize(size)
        })
        resizeObserver.observe(el)

        return () => {
            resizeObserver.disconnect()
        }
    }, [el])
    
    return <const>[setEl, size, el]
}